/**
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * (c) 2015, George Anastassakis (anastas@unipi.gr)
 */
package jpltest;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import jpl.Query;

/**
 * Sample code demonstrating various ways to issue queries and obtain results
 * using SWI Prolog's Java-Prolog interface (JPL).
 */
public class JPLTest {

    public static void main(String[] args) {

		// simply check solvability...
        // make sure the correct path is supplied to consult/1...
		Query q1 = new Query(
            "consult('jpltest.pl')"
        );
        boolean r1 = q1.hasSolution();
        System.out.println("Consult returned: " + r1);

        // obtain a single solution...
        Query q2 = new Query(
            "parent(Parent, Child)"
        );
        Hashtable r2 = q2.oneSolution();
        print(r2);

        // obtain all solutions at once...
        Query q3 = new Query(
            "parent(Parent, Child)"
        );
        Hashtable[] solutions = q3.allSolutions();
        for (int i = 0; i != solutions.length; i++) {
            System.out.println("Solution " + i + " found...");
            Hashtable solution = solutions[i];
            print(solution);
        }

        // iterate over all solutions...
        Query q4 = new Query("parent(Parent, Child)");
        while (q4.hasMoreSolutions()) {
            Hashtable solution = q4.nextSolution();
            print(solution);
            // TODO: Read input from console and
            // continue iteration only if user
            // entered ';'...
        }
    }

    /**
     * Helper method that iterates over a map and prints key-value pairs...
     *
     * @param map
     */
    public static void print(Map map) {
        Iterator i4 = map.entrySet().iterator();
        while (i4.hasNext()) {
            Entry e = (Entry) i4.next();
            System.out.println(e.getKey() + " = " + e.getValue());
        }
    }
}
